#ifndef camera_h_
#define camera_h_

// Canon EOS Shutter-Emulation on Arduino (Nano / Pro Mini, / ...)

//
// Select one of the EOS or one of the EOSM models
//  by uncommenting the corresponding #define:
//
//#define EOS_M
//#define EOS_M3
//#define EOS_M50
// The shutter of the M5 looks like the M50 one (not tested).
//#define EOS_M100
// The shutter of the M10 is probably identical to the M100 one (not tested).

//#define EOS600D	
//#define EOS650D_700D
//#define EOS1100D
//#define EOS1200D_1300D
//#define EOS2000D

//#define EOS6D
//#define EOS6Dii		// 6D Mark 2
//#define EOS50D
//#define EOS60D		// analog to 70D (UNTESTED)
#define EOS60D_alt            // first version of 60D
//#define EOS70D
//#define EOS80D		// 80D without mirror emulation, mirror motor (but not mirror) must be in place
//#define EOS80D_VOLLEMU	//  mirror is also emulated: does not work yet 

//#define EOS200D		// draft only! LV use probably not possible!


// Optional additional defines used to distinguish between cameras
//  with different configurations and/or Arduino pin usage:
// 
//#define MY_600D_3		// eo63a = 600D-Emulator ohne Coil2-Signal --> TRIGGER2_MODE undef.
//#define MY_EOS_M50_1          // M50 with Bluetooth module 
#define MY_2000D_2              // 2000D with Bluetooth module and cooling control

/////////////////////////////////////////////////////////////////////////
// Configuration of interval timer and temperatur sensor.
// These are suggestions only, which may be overwritten for a specific camera.
// 
#ifndef BLUESERIAL_IN_SOFTWARE    // If Bluetooth connection is not predefined:
#define BLUESERIAL_IN_SOFTWARE 1  //  Use SoftwareSerial instead of Serial()
// (Caution: During uploads, the TX line of the Bluetooth module
//  must not be connected to the hardware RX of the Arduino!
#endif

// Arduino pins for interval timer, Bluetooth module connection by
//  SoftwareSerial and NTC temperatur sensor.
// 
// Note: On ATMEGA328P MCUs A6,A7 can not be used as digital I/O lines!
//  If more digital pins are needed, the ATMEGA328PB chip could be used.
//  Or, with MiniCore (https://github.com/MCUdude/MiniCore), the reset pin 
//  can be reconfigured as D22. This, however, will prevent sketch
//   uploads other than via Bluetooth.
//  When using the internal clock, Minicore can make the crystal pins 
//   available as I/O pins D20 and D21.
// 
#ifndef NTC_SENS
#define NTC_SENS      A6      // analogRead() NTC temperature sensor. (The availability of an NTC 
#endif                        //  (must be indicated by the definition of 'T_SENSOR_AVAILABLE')
#ifndef blueRX
#define blueRX        A1      // use A1 for software RX (if desired, --> BLUESERIAL_IN_SOFTWARE)
#endif
#ifndef blueTX
#define blueTX        A2      // use A2 for software TX
#endif
#ifndef SHOOT_LINE
#define SHOOT_LINE    A3      // use A3 to trigger the camera
#endif
                   // A4, A5 might be needed for I2C devices such as the AHT20 sensor module
//

/////////////////////////////////////////////////////////////////////////
// Assignment of the Arduino pin functions to the shutter pins Pnn, where
//  'nn' indicates the position on the shutter cable as indicated in the
//  connection diagrams.
// 
// (The pin definitions reflect the configuration I have used so far.
//  Except for the COILs they can be customised for different requirements.
//  For the coils D2 and/or D3 should be used due are interrupt handling.)
// 
// All connections are made via diodes or 2k protective resistors, so
//  that 5V operation of the Arduino is also possible.
// 
//  Internal  | Arduino |  Shutter-Pin
//   Name     |  D_n    |     Pnn
//
#ifdef EOS_M
#define MOTOR     2     // Pin 15+16:Motor switch (active high)
#define COIL1     3     // Pin 8+9:  Coil switch (active low), Pull-Up notwendig
#define OPT1      5     // Pin 1:    Opto-feedback from motor (LS1)
#define OPT2      6     // Pin 10:   Opto-feedback from shutter coil
                        // Pin 6+7:  Coil Power
                        // Pin 4+11: Gnd
#define EOS_MXXX
#endif 

#ifdef EOS_M3
#define MOTOR     2     // Pin 15+16: Motor Pwr (6V)
#define COIL1     3     // Pin  8:    Coil1 switch (active low), Pull-Up notwendig
#define OPT1      5     // Pin  1:    Opto-feedback from motor rotation, low on startup
#define OPT2      6     // Pin 10:    Opto-feedback from shutter curtain
//               Raw    // Pin  9:    Coil1 Power (4.8V, permanent, 47 Ohm)
//               Gnd    // Pin 11:    Opt2 Gnd
#define EOS_MXXX
#endif 

#ifdef EOS_M50
#define COIL1     2     // Pin 2:  Coil1 switch (active low), Pull-Up notwendig
#define COIL2     3     // Pin 4:  Coil2 switch (active low), Pull-Up notwendig
#define MOTOR     4     // Pin 6,7:  Motor Pwr (+6V)
#define OPT1      5     // Pin 20:   Opto-feedback Blade 1
#define OPT2      6     // Pin 15:   Opto-feedback Blade 2
#define OPTM      7     // Pin 17:   Opto-feedback Motor
#define EOS_MXXX

# ifdef MY_EOS_M50_1     // my M50 with Interval timer
#  undef  SHOOT_LINE              
#  undef  blueRX
#  undef  blueTX
#  define SHOOT_LINE	8	// Ausl�sung, geht an Ausl�ser der Kamera
#  define blueRX 	10	// RX-Leitung zur Anbindung des Bluetooth-Moduls (HC-05):
#  define blueTX 	9	// TX-Leitung zum RX des Bluetooth-Moduls, �ber Vorwiderstand falls 5V-Arduino
# endif 

#endif 

#if defined(EOS_M100)
#define COIL1     3     // Pin 12:  Coil1 switch (active low), Pull-Up notwendig
#define OPT1      5     // Pin  6:  Opto-feedback from motor rotation
#define OPT2      6     // Pin 15:  Opto-feedback from shutter curtain
//               Raw    // Pin 13:  Coil1 Power (4.8V, permanent, 47 Ohm)
//               Gnd    // Pin 14:  Opt2 Gnd
#define EOS_MXXX
#endif



#if defined(EOS6Dii)
# define EOS6D		// 6D und 6Dii unterscheiden sich nur beim Spiegelmotor
#endif

#if defined(EOS6D)

//  Internal  | Arduino |  Shutter-Pin
//   Name     |  D_n    |     Pnn
//
#define COIL2     2     // 9
#define COIL1     3     // 8
#define OPT1      4     // 10: Opto-feedback from shutter coil 1
#define OPT2      8     // 11: Opto-feedback from shutter coil 2
#define ROT1      7     // 2: Rotary feedback 1
#define ROT2      5     // 3: Rotary feedback 2
#define ROTX      6     // 1: Rotary feedback X
#define MR1       9     // 2: Mirror Rot 2 (M2)
#define MR2       10    // 4: Mirror Rot 4 (M4)

#if defined(EOS6Dii)
#else
#define MR3       11    // 5: Mirror Rot 5 (M5)
#endif

#endif

#if defined(EOS60D) || defined(EOS60D_alt)
#define COIL2     2     // 5
#define COIL1     3     // 4
#define OPT1      4     // 7: Opto-feedback from shutter coil 1
#define OPT2      8     // 9: Opto-feedback from shutter coil 2
#define ROT2      5     // 2: Rotary feedback 2
#define ROT3      6     // 3: Rotary feedback 3
#define ROT1      7     // 1: Rotary feedback 1
#define MR1       9     // 5: Mirror Rot S5
#define MR2       10    // 4: Mirror Rot S4
#define MR3       11    // 6: Mirror Rot S6
#endif

#if defined(EOS70D)
#define COIL1     2     // 4
#define COIL2     3     // 5
#define OPT1      4     // 7: Opto-feedback from shutter coil 1
#define OPT2      5     // 9: Opto-feedback from shutter coil 2
#define ROT1      6     // 1: Rotary feedback 1
#define ROT2      7     // 2: Rotary feedback 2
#define ROT3      8     // 3: Rotary feedback 3

#define MR1       9     // 3: Mirror Rot M1
#define MR2       10    // 2: Mirror Rot M2
#define MR3       11    // 1: Mirror Rot M3
#endif

#if defined(EOS80D)
#define COIL1     2     // 4
#define COIL2     3     // 5
#define OPT1      4     // 7: Opto-feedback from shutter coil 1
#define OPT2      5     // 9: Opto-feedback from shutter coil 2
#define ROT1      6     // 1: Rotary feedback 1
#define ROT2      7     // 2: Rotary feedback 2
#define ROT3      8     // 3: Rotary feedback 3

#if defined (EOS80D_VOLLEMU)	    // auch Spiegel wird emuliert:
#define MR1       9     // 28: Mirror M5
#define MR2       10    // 27: Mirror M4
#define MOTORDRV  11    // 25: Mirror Motor Drive M3
#else
#endif // EOS80D_VOLLEMU
#endif

//  Internal  | Arduino |  Shutter-Pin
//   Name     |  D_n    |     Pnn
//
#if defined(EOS50D)
#define COIL2     2     // 4
#define COIL1     3     // 6
#define OPT1      4     // 11: Opto-feedback from shutter coil 1
#define OPT2      5     // 14: Opto-feedback from shutter coil 2
#define ROT1      6     // 1: Rotary feedback 1
#define ROT2      7     // 3: Rotary feedback 2
#define MR1       8     // M3: Mirror Rot 1
#define MR2       9     // M4: Mirror Rot 2
#define MR3       10    // M5: Mirror Rot 3
#endif


#ifdef MY_600D_3
# undef  TRIGGER2_MODE	    // MY_600D_3 hat Coil2-Signal nicht verbunden
#endif

#if defined(EOS600D)
#define COIL2     2     // P1
#define COIL1     3     // P3
#define OPT1      4     // P4: Opto-feedback from shutter coil 1
#define ROT4      5     // P11: Rotary feedback 4
#define ROT1      6     // P12: Rotary feedback 1
#define ROT2      7     // P8:  Rotary feedback 2
#define ROT3      8     // P9:  Rotary feedback 3
#define OPT2      9     // P7: Opto-feedback from shutter coil 2
#endif

#ifdef EOS650D_700D
#define COIL2     2     // P1
#define COIL1     3     // P3
#define OPT1      4     // P4: Opto-feedback from shutter coil 1
#define OPT2      9     // P7: Opto-feedback from shutter coil 2
#define ROT4      5     // P13: Rotary feedback 4
#define ROT1      6     // P14: Rotary feedback 1
#define ROT2      7     // P10: Rotary feedback 2
#define ROT3      8     // P11: Rotary feedback 3
#define ROTX     10     // P9:  Rotary feedback X
#endif 


#if defined(EOS1100D) || defined(EOS1200D_1300D) || defined(EOS2000D)
#define COIL1     3     // P7
#define OPT1      4     // P1: Opto-feedback from shutter
#define ROT4      5     // P8: Rotary feedback 4
#define ROT1      6     // P9: Rotary feedback 1
#define ROT2      7     // P10: Rotary feedback 2
#define ROT3      8     // P12: Rotary feedback 3

#endif

#if defined(EOS2000D) && defined(MY_2000D_2)    
//
// 2000D w/ Bluetooth and cooling control
// !!!May be changed in future versions!!!
// 
# undef  ROT4                            // free D5 for usage as PWM output
# define ROT4                    2       //  and move ROT4 to D2
# undef  SHOOT_LINE              
# define SHOOT_LINE              9

# define T_SENSOR_AVAILABLE              // sensor temperature available (measured via NTC_SENS / A6)
# define COOLER_PWM              5       // PWM signal line (980 Hz on D5) for cooling control
# define COOLER_PWM_PRESET (0.925/3.3*255)  // PWM signal output for standard cooling level. Adjust '3.3' if Vcc is not 3.3V!
# define COOLER_PWM_REVERSE      1       // indicates that the cooling decreases with output signal due to regulation characteristics
# define COOLER_POWER            A3      // additional power switch data line to deaktivate peltier and ventilation
# define COOLER_POWER_ON         HIGH    // peltier power switch ON level
# define HEATER_POWER            10      // power switch data line for window heating
# define HEATER_POWER_ON         HIGH    // heater power switch ON level
# define HEATER_SOFT_PWM                 // apply heater power regulation to HEATER_POWER line by switching on and off once per second
//# define HEATER_PWM                    // apply heater power regulation via an Arduino PWM output (different from HEATER_POWER)
# define HEATER_POWER_BY_COOLER  1       // 1: heater is powered by cooler supply (i.e. off if cooler is off); 2: powered by cooler and fan supply; 0: none

# define AHT20_AVAILABLE                 // measure ambient temperature and humidity via AHT20 sensor module
# define SPIFLASH_CS             A0      // SPI flash memory chip select

# define Ts_CORRECTION           0.1     // corrective addition to the sensor temperature (if not loaded from settings saved in EEPROM)
# define Tu_CORRECTION          -0.9     // corrective addition to the ambient temperature (if not loaded from settings saved in EEPROM)
# define Td_CORRECTION           0.0     // correction of the dew point limitation (if not loaded from settings saved in EEPROM)
                                         // This value is added to the sensor temperatur measure before comparing with calculated due point.

/*/ Testboard:
# undef  AHT20_AVAILABLE                 
//*/

#endif


#endif  // camera_h_
